@@ -2,6 +2,7 @@
 # -*- coding: utf-8 -*-
 # License: BSD-3 (https://tldrlegal.com/license/bsd-3-clause-license-(revised))
 # Copyright (c) 2016-2021, Cabral, Juan; Luczywo, Nadia
+# Copyright (c) 2022, QuatroPe
 # All rights reserved.
 
 # =============================================================================
@@ -16,18 +17,13 @@
 
 from .data import (
     DecisionMatrix,
-    KernelResult,
     Objective,
-    RankResult,
-    ResultABC,
     mkdm,
 )
 from .methods import (
-    SKCDecisionMakerABC,
     SKCMatrixAndWeightTransformerABC,
     SKCMethodABC,
     SKCTransformerABC,
-    SKCWeighterABC,
 )
 from .plot import DecisionMatrixPlotter
 
@@ -39,13 +35,8 @@ __all__ = [
     "mkdm",
     "DecisionMatrix",
     "DecisionMatrixPlotter",
-    "KernelResult",
     "Objective",
-    "RankResult",
-    "ResultABC",
-    "SKCDecisionMakerABC",
     "SKCMatrixAndWeightTransformerABC",
     "SKCMethodABC",
     "SKCTransformerABC",
-    "SKCWeighterABC",
 ]
